"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EError = void 0;
const EMPTY_STACK_TRACE = "<stack trace empty>";
class EError extends Error {
    constructor(messageOrOptionsOrCause, optionsOrCause) {
        var _a, _b;
        let message = "";
        let cause = undefined;
        let info = undefined;
        if (messageOrOptionsOrCause instanceof Error) {
            message = "";
            cause = messageOrOptionsOrCause;
        }
        else if (typeof messageOrOptionsOrCause === "string") {
            message = messageOrOptionsOrCause;
            if (optionsOrCause instanceof Error) {
                cause = optionsOrCause;
            }
            else if (typeof optionsOrCause === "object") {
                cause = optionsOrCause.cause;
                info = optionsOrCause.info;
            }
        }
        else if (typeof messageOrOptionsOrCause === "object") {
            cause = messageOrOptionsOrCause.cause;
            info = messageOrOptionsOrCause.info;
        }
        if (!cause) {
            super(message);
        }
        else {
            const causes = EError.getCauses(cause);
            const messages = [];
            if (message)
                messages.push(message);
            causes.forEach((cause) => {
                const messageString = [cause.name];
                if (cause instanceof EError) {
                    if (cause.originalMessage)
                        messageString.push(cause.originalMessage);
                }
                else {
                    if (cause.message)
                        messageString.push(cause.message);
                }
                messages.push(messageString.join(": "));
            });
            super(messages.join(" > "));
        }
        this.originalMessage = message;
        if (cause)
            this.cause = cause;
        if (info)
            this.info = info;
        Object.setPrototypeOf(this, new.target.prototype);
        this.name = new.target.name;
        (_b = (_a = Error).captureStackTrace) === null || _b === void 0 ? void 0 : _b.call(_a, this, new.target);
    }
    static getCauses(error, filter) {
        const causes = [];
        let cause = error;
        while (cause) {
            if (!filter || filter(cause))
                causes.push(cause);
            cause = cause.cause instanceof Error ? cause.cause : null;
        }
        return causes;
    }
    getCauses(filter) {
        return EError.getCauses(this, filter);
    }
    static findCause(error, type) {
        let cause = error;
        while (cause) {
            if (cause instanceof type && cause.name === type.name)
                return cause;
            cause = cause.cause instanceof Error ? cause.cause : null;
        }
        return null;
    }
    findCause(type) {
        return EError.findCause(this, type);
    }
    static findCauses(error, type) {
        return EError.getCauses(error, (e) => e instanceof type && e.name === type.name);
    }
    findCauses(type) {
        return EError.getCauses(this, (e) => e instanceof type && e.name === type.name);
    }
    static findCauseByName(error, name) {
        let cause = error;
        while (cause) {
            if (cause instanceof Error && cause.name === name)
                return cause;
            cause = cause.cause instanceof Error ? cause.cause : null;
        }
        return null;
    }
    findCauseByName(name) {
        return EError.findCauseByName(this, name);
    }
    static findCausesByName(error, name) {
        return EError.getCauses(error, (e) => e.name === name);
    }
    findCausesByName(name) {
        return EError.findCausesByName(this, name);
    }
    static fullStack(error) {
        var _a;
        let stack = (_a = error.stack) !== null && _a !== void 0 ? _a : EMPTY_STACK_TRACE;
        if (error.cause instanceof Error)
            stack += "\n\n" + `caused by: ${EError.fullStack(error.cause)}`;
        return stack;
    }
    fullStack() {
        return EError.fullStack(this);
    }
    static toJSON(error, options = {}) {
        const { stack, shallow } = options;
        const json = {
            name: error.name,
            message: error.message,
        };
        if (error instanceof EError && typeof error.originalMessage === "string") {
            json.originalMessage = error.originalMessage;
        }
        if (!shallow && error.cause instanceof Error) {
            json.cause = EError.toJSON(error.cause, options);
        }
        if (error instanceof EError && error.info) {
            json.info = error.info;
        }
        if (stack) {
            const stack = error.stack;
            if (stack)
                json.stack = stack.split("\n");
        }
        return json;
    }
    toJSON(options = {}) {
        return EError.toJSON(this, options);
    }
}
exports.EError = EError;
